<?php

return array(

	/*
	 * The Authentication library to use
	 * Make sure that the library supports:
	 * 1) A get_user method that returns FALSE when no user is logged in
	 *	  and a user object that implements Acl_Role_Interface when a user is logged in
	 * 2) A static instance method to instantiate a Authentication object
	 *
	 * array(CLASS_NAME,array $arguments)
	 */
	'lib' => array(
        'class'  => 'AUTH', // (or AUTH)
        'params' => array('auth')
    ),

	'exception' => FALSE,

	/*
	 * The ACL Roles (String IDs are fine, use of ACL_Role_Interface objects also possible)
	 * Use: ROLE => PARENT(S) (make sure parent is defined as role itself before you use it as a parent)
	 */
	'roles' => array
	(
	    'author' => 'guest',
		'admin'  => 'author'
	),

	/*
	 * The name of the guest role 
	 * Used when no user is logged in.
	 */
	'guest_role' => 'guest',

	/*
	 * The ACL Resources (String IDs are fine, use of ACL_Resource_Interface objects also possible)
	 * Use: ROLE => PARENT (make sure parent is defined as resource itself before you use it as a parent)
	 */
	'resources' => array
	(
		'blog' => NULL
	),

	/*
	 * The ACL Rules (Again, string IDs are fine, use of ACL_Role/Resource_Interface objects also possible)
	 * Split in allow rules and deny rules, one sub-array per rule:
	     array( ROLES, RESOURCES, PRIVILEGES, ASSERTION)
	 *
	 * Assertions are defined as follows :
			array(CLASS_NAME,$argument) // (only assertion objects that support (at most) 1 argument are supported
			                            //  if you need to give your assertion object several arguments, use an array)
	 */
	'rules' => array
	(
		'allow' => array
		(
			// guest can read blog
			'guest1' => array(
				'role'      => 'guest',
				'resource'  => 'blog',
				'privilege' => 'read'
			),

			// authors can add blogs
			'author1' => array(
				'role'      => 'author',
				'resource'  => 'blog',
				'privilege' => 'add'
			),

			// authors can edit & delete their own blogs (and only their own blogs)
			'author2' => array(
				'role'      => 'author',
				'resource'  => 'blog',
				'privilege' => array('edit','delete'),
				'assertion' => array('Acl_Assert_Argument',array('id'=>'user_id'))
			),

			// administrators can delete and edit everything 
			'admin1' => array(
				'role'      => 'admin',
				'resource'  => 'blog',
				'privilege' => array('delete', 'edit')
			),
		),
		'deny' => array
		(
			  // no deny rules in this example
		)
	)
);